@@ -62,8 +62,6 @@
 #include <xen/platform-compat.h>
 #endif
 
-#define VMDQ_VNIC "vmdq_vnic"
-
 int xen_store_evtchn;
 struct xenstore_domain_interface *xen_store_interface;
 static unsigned long xen_store_mfn;
@@ -138,12 +136,6 @@ static void free_otherend_watch(struct xenbus_device *dev)
 int read_otherend_details(struct xenbus_device *xendev,
 				 char *id_node, char *path_node)
 {
-
-    /* add for vmdq migrate.When the device is vmdq_vnic ,return */
-	if(0 == strcmp(xendev->devicetype, VMDQ_VNIC)){
-        return 0;
-	}
-    
 	int err = xenbus_gather(XBT_NIL, xendev->nodename,
 				id_node, "%i", &xendev->otherend_id,
 				path_node, NULL, &xendev->otherend,
@@ -247,7 +239,7 @@ static void otherend_changed(struct xenbus_watch *watch,
 	 * work that can fail e.g., when the rootfs is gone.
 	 */
 	if (system_state > SYSTEM_RUNNING) {
-		struct xen_bus_type *bus = bus;
+		struct xen_bus_type *bus = NULL;
 		bus = container_of(dev->dev.bus, struct xen_bus_type, bus);
 		/* If we're frontend, drive the state machine to Closed. */
 		/* This should cause the backend to release our resources. */
@@ -274,10 +266,6 @@ static int talk_to_otherend(struct xenbus_device *dev)
 
 static int watch_otherend(struct xenbus_device *dev)
 {
-    /* add for vmdq migrate.When the device is vmdq_vnic ,return */
-	if(0 == strcmp(dev->devicetype, VMDQ_VNIC)){
-        return 0;
-	}
 	return xenbus_watch_path2(dev, dev->otherend, "state",
 				  &dev->otherend_watch, otherend_changed);
 }
@@ -1065,11 +1053,6 @@ static int is_disconnected_device(struct device *dev, void *data)
 
 static int exists_disconnected_device(struct device_driver *drv)
 {
-    /* add for vmdq migrate.When the device is vmdq_vnic ,return */
-	if(0 == strcmp(drv->name, VMDQ_VNIC)){
-        return 0;
-	}
-    
 	if (xenbus_frontend.error)
 		return xenbus_frontend.error;
 	return bus_for_each_dev(&xenbus_frontend.bus, NULL, drv,
