@@ -218,7 +218,7 @@ unsigned long balloon_num_physpages(void)
 	unsigned long phys_pages = 0;
 
 	for_each_online_node(nid)
-		phys_pages += node_spanned_pages(nid);
+		phys_pages += node_present_pages(nid);
 
 	return phys_pages;
 }
@@ -593,19 +593,6 @@ static int __init balloon_init(void)
 	 * extent of 1. When start_extent > nr_extents (>= in newer Xen), we
 	 * simply get start_extent returned.
 	 */
-	#if 0
-	totalram_bias = HYPERVISOR_memory_op(rc != -ENOSYS && rc != 1
-		? XENMEM_maximum_reservation : XENMEM_current_reservation,
-		&pod_target.domid);
-	if ((long)totalram_bias != -ENOSYS) {
-		BUG_ON(totalram_bias < totalram_pages);
-		bs.current_pages = totalram_bias;
-		totalram_bias -= totalram_pages;
-	} else {
-		totalram_bias = 0;
-		bs.current_pages = totalram_pages;
-	}
-	#endif
 	bs.current_pages = pod_target.tot_pages + pod_target.pod_entries
 			   - pod_target.pod_cache_pages;
 	if (rc || bs.current_pages > num_physpages)
